@@ -6,8 +6,8 @@
 //   LineStripSet
 //
 
-LineStripSet::LineStripSet(Material& in_material, int in_nvertices, double* in_vertex, int in_ignoreExtent)
-  : PrimitiveSet(in_material, in_nvertices, in_vertex, GL_LINE_STRIP, 1, in_ignoreExtent)
+LineStripSet::LineStripSet(Material& in_material, int in_nvertices, double* in_vertex, bool in_ignoreExtent, bool in_bboxChange)
+  : PrimitiveSet(in_material, in_nvertices, in_vertex, GL_LINE_STRIP, 1, in_ignoreExtent, in_bboxChange)
 {
   material.lit = false;
   if (material.line_antialias) blended = true;
