@@ -8,7 +8,7 @@
 //   RenderContext
 //
 
-float RenderContext::getDistance(const Vertex& v)
+float RenderContext::getDistance(const Vertex& v) const
 {
   Vertex4 vec = Vertex4(v, 1.0f);
 
